@@ -4,7 +4,10 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
+#include "testhelper.h"
+
 #include <timelinedelegatecontroller.h>
+#include <applicationcontroller.h>
 #include <livedatamanager.h>
 #include <reservationmanager.h>
 
@@ -27,15 +30,6 @@ class TimelineDelegateControllerTest : public QObject
 {
     Q_OBJECT
 private:
-    void clearReservations(ReservationManager *mgr)
-    {
-        const auto batches = mgr->batches(); // copy, as this is getting modified in the process
-        for (const auto &id : batches) {
-            mgr->removeBatch(id);
-        }
-        QCOMPARE(mgr->batches().size(), 0);
-    }
-
     QByteArray readFile(const QString &fn)
     {
         QFile f(fn);
@@ -82,7 +76,7 @@ private Q_SLOTS:
     void testProgress()
     {
         ReservationManager mgr;
-        clearReservations(&mgr);
+        Test::clearAll(&mgr);
 
         TrainTrip trip;
         trip.setTrainNumber(QStringLiteral("TGV 1235"));
@@ -119,7 +113,7 @@ private Q_SLOTS:
     void testPreviousLocation()
     {
         ReservationManager mgr;
-        clearReservations(&mgr);
+        Test::clearAll(&mgr);
 
         TimelineDelegateController controller;
         controller.setReservationManager(&mgr);
@@ -162,12 +156,16 @@ private Q_SLOTS:
     void testJourneyRequest()
     {
         ReservationManager mgr;
-        clearReservations(&mgr);
-        mgr.importReservation(readFile(QLatin1String(SOURCE_DIR "/../tests/randa2017.json")));
+        Test::clearAll(&mgr);
+        LiveDataManager ldm;
+        ApplicationController ctrl;
+        ctrl.setReservationManager(&mgr);
+        ctrl.importFromUrl(QUrl::fromLocalFile(QLatin1String(SOURCE_DIR "/../tests/randa2017.json")));
 
         TimelineDelegateController controller;
         controller.setReservationManager(&mgr);
         controller.setBatchId(mgr.batches().at(0)); // flight
+        controller.setLiveDataManager(&ldm);
         QVERIFY(!controller.journeyRequest().isValid());
 
         controller.setBatchId(mgr.batches().at(1)); // first train segment
@@ -189,11 +187,13 @@ private Q_SLOTS:
     void testApplyJourney()
     {
         ReservationManager mgr;
-        clearReservations(&mgr);
+        Test::clearAll(&mgr);
         LiveData::clearStorage();
         LiveDataManager ldm;
         ldm.setReservationManager(&mgr);
-        mgr.importReservation(readFile(QLatin1String(SOURCE_DIR "/../tests/randa2017.json")));
+        ApplicationController ctrl;
+        ctrl.setReservationManager(&mgr);
+        ctrl.importFromUrl(QUrl::fromLocalFile(QLatin1String(SOURCE_DIR "/../tests/randa2017.json")));
 
         TimelineDelegateController controller;
         controller.setReservationManager(&mgr);
@@ -232,8 +232,10 @@ private Q_SLOTS:
     void testCancel()
     {
         ReservationManager mgr;
-        clearReservations(&mgr);
-        mgr.importReservation(readFile(QLatin1String(SOURCE_DIR "/data/timeline/flight-cancelation.json")));
+        Test::clearAll(&mgr);
+        ApplicationController ctrl;
+        ctrl.setReservationManager(&mgr);
+        ctrl.importFromUrl(QUrl::fromLocalFile(QLatin1String(SOURCE_DIR "/data/timeline/flight-cancelation.json")));
 
         TimelineDelegateController controller;
         controller.setReservationManager(&mgr);
