@@ -16,13 +16,10 @@
 
 #include <KLocalizedString>
 
-#include <QClipboard>
 #include <QDebug>
 #include <QDir>
 #include <QDirIterator>
 #include <QFile>
-#include <QGuiApplication>
-#include <QMimeData>
 #include <QStandardPaths>
 #include <QUuid>
 
@@ -48,19 +45,19 @@ static QString basePath()
     return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + QLatin1String("/health-certificates/");
 }
 
-void HealthCertificateManager::importCertificate(const QByteArray &rawData)
+bool HealthCertificateManager::importCertificate(const QByteArray &rawData)
 {
     // check whether we know this certificate already
     for (const auto &c : m_certificates) {
         if (certificateRawData(c) == rawData) {
-            return;
+            return true;
         }
     }
 #if HAVE_KHEALTHCERTIFICATE
     CertData certData;
     certData.cert = KHealthCertificateParser::parse(rawData);
     if (certData.cert.isNull()) {
-        return;
+        return false;
     }
 
     auto path = basePath();
@@ -71,7 +68,7 @@ void HealthCertificateManager::importCertificate(const QByteArray &rawData)
     QFile f(path);
     if (!f.open(QFile::WriteOnly)) {
         qWarning() << f.errorString() << f.fileName();
-        return;
+        return false;
     }
     f.write(rawData);
     f.close();
@@ -81,19 +78,13 @@ void HealthCertificateManager::importCertificate(const QByteArray &rawData)
     beginInsertRows({}, row, row);
     m_certificates.insert(it, std::move(certData));
     endInsertRows();
+    Q_EMIT newCertificateLoaded(row);
+    return true;
+#else
+    return false;
 #endif
 }
 
-void HealthCertificateManager::importCertificateFromClipboard()
-{
-    const auto md = QGuiApplication::clipboard()->mimeData();
-    if (md->hasText()) {
-        importCertificate(md->text().toUtf8());
-    } else if (md->hasFormat(QLatin1String("application/octet-stream"))) {
-        importCertificate(md->data(QLatin1String("application/octet-stream")));
-    }
-}
-
 void HealthCertificateManager::removeCertificate(int row)
 {
     beginRemoveRows({}, row, row);
